package com.wechi.adweb.bridge.google.analytics.controller;

import com.google.gson.Gson;
import com.wechi.adweb.bridge.common.APIStatus;
import com.wechi.adweb.bridge.common.BaseController;
import com.wechi.adweb.bridge.common.OpenAPIRequest;
import com.wechi.adweb.bridge.common.OpenAPIResponse;
import com.wechi.adweb.bridge.exception.BadRequestException;
import com.wechi.adweb.bridge.exception.DataException;
import com.wechi.adweb.bridge.google.analytics.dto.GAAccountDTO;
import com.wechi.adweb.bridge.google.analytics.dto.GAPropertyDTO;
import com.wechi.adweb.bridge.google.analytics.dto.report.GAReportRequestDTO;
import com.wechi.adweb.bridge.google.analytics.dto.report.ReportType;
import com.wechi.adweb.bridge.google.analytics.dto.report.data.GAReportDataDTO;
import com.wechi.adweb.bridge.google.analytics.service.GAAdminService;
import com.wechi.adweb.bridge.google.analytics.service.GADataService;

import io.swagger.v3.oas.annotations.Operation;

import lombok.extern.slf4j.Slf4j;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

/**
 * @author wfansh
 */
@Slf4j
@RestController
@RequestMapping("/api/google/ga")
public class GAController extends BaseController {
    @Autowired private GADataService gaDataService;

    @Autowired private GAAdminService gaAdminService;

    private final Gson gson = new Gson();

    @PostMapping("/accounts/list")
    @ResponseBody
    public OpenAPIResponse<List<GAAccountDTO>> listGAAccounts() throws DataException {
        long start = System.currentTimeMillis();
        log.info("****** listGAAccounts() ******");

        List<GAAccountDTO> gaAccounts = gaAdminService.listGAAccounts();

        log.info("****** listGAAccounts() ****** duration = {} seconds", getElapsedSeconds(start));
        return OpenAPIResponse.<List<GAAccountDTO>>builder()
                .status(APIStatus.SUCCESS)
                .data(gaAccounts)
                .build();
    }

    @Operation(summary = "List the properties under a GA account name.")
    @PostMapping("/properties/list")
    @ResponseBody
    public OpenAPIResponse<List<GAPropertyDTO>> listGAProperties(
            @RequestBody OpenAPIRequest<String> apiRequest)
            throws BadRequestException, DataException {
        long start = System.currentTimeMillis();
        log.info("****** listGAProperties() ****** apiRequest = {}", gson.toJson(apiRequest));

        // 1. Validates request parameters - GA account name.
        if (StringUtils.isEmpty(apiRequest.getData())) {
            throw new BadRequestException(apiRequest);
        }

        // 2. Executes the API request.
        List<GAPropertyDTO> gaProperties =
                gaAdminService.listGAProperties(
                        GAAccountDTO.getResourceName(apiRequest.getData()), false);
        log.info(
                "****** listGAProperties() ****** duration = {} seconds", getElapsedSeconds(start));
        return OpenAPIResponse.<List<GAPropertyDTO>>builder()
                .status(APIStatus.SUCCESS)
                .data(gaProperties)
                .build();
    }

    @PostMapping("/report")
    @ResponseBody
    public OpenAPIResponse<List<? extends GAReportDataDTO>> getGAReport(
            @RequestBody OpenAPIRequest<GAReportRequestDTO> apiRequest)
            throws BadRequestException, DataException {
        long start = System.currentTimeMillis();
        log.info("****** getGAReport() ****** apiRequest = {}", gson.toJson(apiRequest));
        GAReportRequestDTO reportRequest = apiRequest.getData();

        // 1. Validates request parameters.
        if (Objects.isNull(reportRequest)
                || StringUtils.isEmpty(reportRequest.getPropertyName())
                || Objects.isNull(reportRequest.getReportType())
                || StringUtils.isEmpty(reportRequest.getStartDate())
                || StringUtils.isEmpty(reportRequest.getEndDate())
                || (ReportType.ADWEB_CUSTOM_REPORT.equals(reportRequest.getReportType())
                        && CollectionUtils.isEmpty(reportRequest.getMetrics())
                        && CollectionUtils.isEmpty(reportRequest.getDimensions()))) {
            throw new BadRequestException(apiRequest);
        }

        // 2. Executes the API request.
        List<? extends GAReportDataDTO> report = gaDataService.runReport(apiRequest.getData());
        log.info("****** getGAReport() ****** duration = {} seconds", getElapsedSeconds(start));
        return OpenAPIResponse.<List<? extends GAReportDataDTO>>builder()
                .status(APIStatus.SUCCESS)
                .data(report)
                .build();
    }
}
