package com.wechi.adweb.bridge.google.analytics.service;

import com.google.analytics.admin.v1beta.*;
import com.google.api.client.util.Lists;
import com.google.api.gax.core.FixedCredentialsProvider;
import com.google.auth.oauth2.GoogleCredentials;
import com.google.common.base.Splitter;
import com.google.gson.Gson;
import com.google.protobuf.util.JsonFormat;
import com.wechi.adweb.bridge.exception.DataException;
import com.wechi.adweb.bridge.google.analytics.dto.GAAccountDTO;
import com.wechi.adweb.bridge.google.analytics.dto.GADataStreamDTO;
import com.wechi.adweb.bridge.google.analytics.dto.GAPropertyDTO;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import javax.annotation.PostConstruct;

/**
 * @author wfansh
 */
@Slf4j
@Service
public class GAAdminService {

    private static final String DEFAULT_TIMEZONE = "Asia/Shanghai";

    @Value("${google.service.account.key}")
    private String serviceAccountKey;

    private AnalyticsAdminServiceSettings adminServiceSettings;

    private final Gson gson = new Gson();

    @PostConstruct
    private void init() throws IOException {
        GoogleCredentials credentials =
                GoogleCredentials.fromStream(
                        this.getClass().getClassLoader().getResourceAsStream(serviceAccountKey));

        this.adminServiceSettings =
                AnalyticsAdminServiceSettings.newBuilder()
                        .setCredentialsProvider(FixedCredentialsProvider.create(credentials))
                        .build();
    }

    public List<GAAccountDTO> listGAAccounts() throws DataException {
        List<GAAccountDTO> gaAccounts = Lists.newArrayList();
        try (AnalyticsAdminServiceClient analyticsAdminServiceClient =
                AnalyticsAdminServiceClient.create(adminServiceSettings)) {
            // Initializes the list request with default settings.
            ListAccountsRequest request = ListAccountsRequest.getDefaultInstance();

            for (Account account : analyticsAdminServiceClient.listAccounts(request).iterateAll()) {
                GAAccountDTO gaAccount =
                        GAAccountDTO.builder()
                                .id(parseId(account.getName()))
                                .name(account.getName())
                                .displayName(account.getDisplayName())
                                .createTime(account.getCreateTime().getSeconds())
                                .updateTime(account.getUpdateTime().getSeconds())
                                .build();

                // Adds into the list;
                gaAccounts.add(gaAccount);
            }

            log.info("listGAAccounts : {}", gson.toJson(gaAccounts));
            return gaAccounts;
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new DataException(e);
        }
    }

    public List<GAPropertyDTO> listGAProperties(String accountName, boolean withDataStreams)
            throws DataException {
        List<GAPropertyDTO> gaProperties = Lists.newArrayList();
        try (AnalyticsAdminServiceClient analyticsAdminServiceClient =
                AnalyticsAdminServiceClient.create(adminServiceSettings)) {
            // Initializes the list request with account name filter.
            // See
            // https://developers.google.com/analytics/devguides/config/admin/v1/rest/v1beta/properties/list#http-request
            ListPropertiesRequest request =
                    ListPropertiesRequest.newBuilder().setFilter("ancestor:" + accountName).build();

            for (Property property :
                    analyticsAdminServiceClient.listProperties(request).iterateAll()) {
                GAPropertyDTO gaProperty =
                        GAPropertyDTO.builder()
                                .id(parseId(property.getName()))
                                .name(property.getName())
                                .displayName(property.getDisplayName())
                                .propertyType(property.getPropertyType().toString())
                                .account(property.getAccount())
                                .parent(property.getParent())
                                .industryCategory(property.getIndustryCategory().toString())
                                .timeZone(property.getTimeZone())
                                .currencyCode(property.getCurrencyCode())
                                .serviceLevel(property.getServiceLevel().toString())
                                .createTime(property.getCreateTime().getSeconds())
                                .updateTime(property.getUpdateTime().getSeconds())
                                // Sets the data streams associated with this property.
                                .dataStreams(
                                        withDataStreams
                                                ? listGADataStreams(property.getName())
                                                : Collections.EMPTY_LIST)
                                .build();

                // Adds into the list;
                gaProperties.add(gaProperty);
            }

            // Distinct by property name.
            gaProperties =
                    gaProperties.stream()
                            .collect(Collectors.groupingBy(GAPropertyDTO::getName))
                            .values()
                            .stream()
                            .map(values -> values.get(0))
                            .toList();

            log.info("listGAProperties for {} : {}", accountName, gson.toJson(gaProperties));
            return gaProperties;
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new DataException(e);
        }
    }

    public List<GADataStreamDTO> listGADataStreams(String propertyName) throws DataException {
        List<GADataStreamDTO> gaDataStreams = Lists.newArrayList();
        try (AnalyticsAdminServiceClient analyticsAdminServiceClient =
                AnalyticsAdminServiceClient.create(adminServiceSettings)) {
            // Initializes the list request with parent property.
            ListDataStreamsRequest request =
                    ListDataStreamsRequest.newBuilder().setParent(propertyName).build();

            for (DataStream dataStream :
                    analyticsAdminServiceClient.listDataStreams(request).iterateAll()) {
                GADataStreamDTO gaDataStream =
                        GADataStreamDTO.builder()
                                .id(parseId(dataStream.getName()))
                                .name(dataStream.getName())
                                .displayName(dataStream.getDisplayName())
                                .property(propertyName)
                                .streamMeasurementId(
                                        dataStream.getWebStreamData().getMeasurementId())
                                .streamDefaultUrl(dataStream.getWebStreamData().getDefaultUri())
                                .build();

                // Adds into the list;
                gaDataStreams.add(gaDataStream);
            }

            log.info("listGADataStreams for {} : {}", propertyName, gson.toJson(gaDataStreams));
            return gaDataStreams;
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new DataException(e);
        }
    }

    public GADataStreamDTO createGAPropertyWithDataStream(
            String accountName, String displayName, String url) throws DataException {
        try (AnalyticsAdminServiceClient analyticsAdminServiceClient =
                AnalyticsAdminServiceClient.create(adminServiceSettings)) {
            // 1. Creates the property.
            Property property =
                    Property.newBuilder()
                            .setParent(accountName)
                            .setDisplayName(displayName)
                            .setTimeZone(DEFAULT_TIMEZONE)
                            .build();
            Property createdProperty = analyticsAdminServiceClient.createProperty(property);
            log.info("Created property {}", JsonFormat.printer().print(createdProperty));

            // 2. Creates the data stream.
            DataStream dataStream =
                    DataStream.newBuilder()
                            .setDisplayName(displayName)
                            .setType(DataStream.DataStreamType.WEB_DATA_STREAM)
                            .setWebStreamData(
                                    DataStream.WebStreamData.newBuilder().setDefaultUri(url))
                            .build();
            DataStream createdDataStream =
                    analyticsAdminServiceClient.createDataStream(
                            createdProperty.getName(), dataStream);
            log.info("Created data stream {}", JsonFormat.printer().print(createdProperty));

            GADataStreamDTO gaDataStream =
                    GADataStreamDTO.builder()
                            .id(parseId(createdDataStream.getName()))
                            .name(createdDataStream.getName())
                            .displayName(createdDataStream.getDisplayName())
                            .property(createdProperty.getName())
                            .streamMeasurementId(
                                    createdDataStream.getWebStreamData().getMeasurementId())
                            .streamDefaultUrl(createdDataStream.getWebStreamData().getDefaultUri())
                            .build();

            return gaDataStream;
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new DataException(e);
        }
    }

    /** Parses GA ID from the resource name. */
    private static String parseId(String name) {
        List<String> parts = Splitter.on('/').splitToList(name);
        return parts.size() == 2 ? parts.get(1) : null;
    }
}
